#include "graph_lib.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <math.h>

#define LOG_DEBUG 1
#define LOG_WARN 2
#define LOG_ERR 3
#define LOG_LEVEL LOG_WARN


void set_prefix( coloring_t *coloring, int num_vertices_in_prefix, int num_edges, int *indexes ) {
	int combin[10][3] = {
		{ 0, 1, 2 },
		{ 0, 1, 3 },
		{ 0, 1, 4 },
		{ 0, 2, 3 },
		{ 0, 2, 4 },
		{ 0, 3, 4 },
		{ 1, 2, 3 },
		{ 1, 2, 4 },
		{ 1, 3, 4 },
		{ 2, 3, 4 },
	};
	
	int i, j;
	
	for ( i = 0; i < num_vertices_in_prefix; i++ ) {
		for ( j = 0; j < 3; j++ ) {
			coloring->colors[3*i+j] = combin[indexes[i]][j];
		}
	}
	
	for ( i = num_vertices_in_prefix * 3; i < num_edges; i++ ) {
		coloring->colors[i] = -1;
	}
}





int main(int argc, char *argv[])
{

	
	
	graph_t *graph = g_create( 17 );

	g_add_edge( graph, 0, 1 );
	g_add_edge( graph, 0, 2 );
	g_add_edge( graph, 0, 3 );
	g_add_edge( graph, 0, 4 );
	g_add_edge( graph, 1, 5 );
	g_add_edge( graph, 1, 6 );
	g_add_edge( graph, 1, 7 );
	g_add_edge( graph, 2, 8 );
	g_add_edge( graph, 2, 9 );
	g_add_edge( graph, 2, 10 );
	g_add_edge( graph, 3, 11 );
	g_add_edge( graph, 3, 12 );
	g_add_edge( graph, 3, 13 );
	g_add_edge( graph, 4, 14 );
	g_add_edge( graph, 4, 15 );
	g_add_edge( graph, 4, 16 );
	

	//enumerate graph so that central vertex would be the last
	
	g_set_data_both_directions( graph, 0, 1, 5 );
	g_set_data_both_directions( graph, 1, 1, 6 );
	g_set_data_both_directions( graph, 2, 1, 7 );
	g_set_data_both_directions( graph, 3, 2, 8 );
	g_set_data_both_directions( graph, 4, 2, 9 );
	g_set_data_both_directions( graph, 5, 2, 10 );
	g_set_data_both_directions( graph, 6, 3, 11 );
	g_set_data_both_directions( graph, 7, 3, 12 );
	g_set_data_both_directions( graph, 8, 3, 13 );
	g_set_data_both_directions( graph, 9, 4, 14 );
	g_set_data_both_directions( graph, 10, 4, 15 );
	g_set_data_both_directions( graph, 11, 4, 16 );
	g_set_data_both_directions( graph, 12, 0, 1 );
	g_set_data_both_directions( graph, 13, 0, 2 );
	g_set_data_both_directions( graph, 14, 0, 3 );
	g_set_data_both_directions( graph, 15, 0, 4 );
	
	graph_t *linegraph = g_create_linegraph_from_enumerated_graph( graph, 16 );
	graph_t *graph_of_colorings;	

	coloring_t *coloring = g_create_coloring( 5, linegraph->num_vertices );
	
	code_properties_t code_properties = g_code_properties( 5, linegraph->num_vertices ); //num_colors = 5

	
	int num_colorings=0, number_of_non_kempe_equivalent = 0;

	int i, j, k, l;
	char **dictionary;
	
	for ( i = 0; i<10; i++ ) {
		for (j = 0; j<10; j++ ) {
			for (k = 0; k<10; k++) {
				for (l = 0; l<10; l++) {
					int indexes[4] = {i, j, k, l};	
					
					set_prefix( coloring, 4, 16, indexes );

					num_colorings = 0;
					//the coloring is correct when the colors of edges around central vertex are 0,1,2,3
					int correct_coloring = 0;

					while( g_get_next_coloring_offset( linegraph, coloring, 12 ) ) {
						num_colorings++;
						//since central vertex edges are the last ones, we test edges 12,13,14 and 15
						if ( ( coloring->colors[12] == 0 ) &&
							( coloring->colors[13] == 1 ) &&
							( coloring->colors[14] == 2 ) &&
							( coloring->colors[15] == 3 ) ) {
							correct_coloring = 1;
						}
					};

					if (num_colorings < 2 || correct_coloring == 0) continue;

					set_prefix( coloring, 4, 16, indexes );
					
					dictionary = g_create_dictionary( num_colorings );
					graph_of_colorings = g_create( num_colorings );

										
					num_colorings=0;
					
					
					while ( g_get_next_coloring_offset( linegraph, coloring, 12 ) ) {
							
						dictionary[num_colorings] = g_create_code( &code_properties );
						g_coloring_to_code( coloring, dictionary[num_colorings], &code_properties );
						num_colorings++;
					}	

					int m;
					//perform switching
					for (m=0; m<num_colorings; m++) {
						g_code_to_coloring( dictionary[m], coloring, &code_properties );
						perform_kempe_switching_on_coloring( linegraph, graph_of_colorings, dictionary, coloring, 12 );
					}
					
					nv_t nc = g_num_components( graph_of_colorings );
					
					if (nc > 1) {
						fprintf( stdout, "num_compontents = %lu\n", nc );
						set_prefix( coloring, 4, 16, indexes );
						g_dump_tgf_edgecoloring( graph, coloring );

//						g_dump_coloring( coloring );
						number_of_non_kempe_equivalent++;
					}
					
					//destroy dictionary and goc - avoid memory leak
					for (m=0; m<num_colorings; m++) {
						free(dictionary[m]);
					}
					g_destroy_dictionary( &dictionary );
					
					g_destroy( &graph_of_colorings );
					
					//and let's go to another coloring prefix
				}
			}
		}
	}
		
	printf("number_of_non_kempe_equivalent = %d\n", number_of_non_kempe_equivalent );

	g_destroy( &graph );
	g_destroy( &linegraph );
	g_destroy_coloring( &coloring );

	
	
	
	
	return 0;

}
