#include "graph_lib.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <math.h>

#define LOG_DEBUG 1
#define LOG_WARN 2
#define LOG_ERR 3
#define LOG_LEVEL LOG_WARN


int combin[10][3] = {
	{ 0, 1, 2 },
	{ 0, 1, 3 },
	{ 0, 1, 4 },
	{ 0, 2, 3 },
	{ 0, 2, 4 },
	{ 0, 3, 4 },
	{ 1, 2, 3 },
	{ 1, 2, 4 },
	{ 1, 3, 4 },
	{ 2, 3, 4 },
};


int complement[10][2] = {
	{ 3, 4 },
	{ 2, 4 },
	{ 2, 3 },
	{ 1, 4 },
	{ 1, 3 },
	{ 1, 2 },
	{ 0, 4 },
	{ 0, 3 },
	{ 0, 2 },
	{ 0, 1 },
};

void set_prefix( coloring_t *coloring, int num_vertices_in_prefix, int num_edges, int *indexes ) {
	int i, j;
	
	for ( i = 0; i < num_vertices_in_prefix; i++ ) {
		for ( j = 0; j < 3; j++ ) {
			coloring->colors[3*i+j] = combin[indexes[i]][j];
		}
	}
	
	for ( i = num_vertices_in_prefix * 3; i < num_edges; i++ ) {
		coloring->colors[i] = -1;
	}
}


void g_dump_goc_mg_graphml( graph_t *graph,
						graph_t *graph_of_colorings,
						graph_t *linegraph, 
						char **dictionary,
						int num_colors,
						int *indexes ) {

	//print header
	printf( "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" );
	printf( "<graphml xmlns=\"http://graphml.graphdrawing.org/xmlns\"\n" );
	printf( "         xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n" );  
	printf( "         xsi:schemaLocation=\"http://graphml.graphdrawing.org/xmlns\n" );  
	printf( "         http://www.yworks.com/xml/schema/graphml/1.1/ygraphml.xsd\"\n" );  
	printf( "         xmlns:y=\"http://www.yworks.com/xml/graphml\">\n" );  
	printf( "  <key id=\"d0\" for=\"node\" yfiles.type=\"nodegraphics\"/>  \n" );
	printf( "  <key id=\"d1\" for=\"edge\" yfiles.type=\"edgegraphics\"/>  \n" );
	printf( "  <graph id=\"G\" edgedefault=\"undirected\"> \n" );

	
	code_properties_t properties = g_code_properties( num_colors, linegraph->num_vertices );
	coloring_t *coloring = g_create_coloring( num_colors, linegraph->num_vertices);
	
	//print nodes
	nv_t i;
	for ( i=0; i<graph_of_colorings->num_vertices; i++ ) {
		printf("    <node id=\"n%lu\" yfiles.foldertype=\"folder\">\n", i );
		printf("      <data key=\"d0\">                                   \n" );
		printf("        <y:GroupNode>                                   \n" );
		printf("          <y:NodeLabel>%lu</y:NodeLabel>\n \n", i );
		printf("          <y:Fill color=\"#eeeeee\" transparent=\"false\"/> \n" );
		printf("        </y:GroupNode>                                  \n" );
		printf("      </data>                                           \n" );

		g_erase_coloring( coloring );
		g_code_to_coloring( dictionary[i], coloring, &properties );

		//print a matt's graph
		graph_t *mgraph = g_create( 10 );
		
		g_add_edge( mgraph, complement[indexes[0]][0], complement[indexes[0]][1] );
		g_add_edge( mgraph, complement[indexes[1]][0], complement[indexes[1]][1] );
		g_add_edge( mgraph, complement[indexes[2]][0], complement[indexes[2]][1] );
		
		int j, used[5], vertex1=-1, vertex2=-1;
		for (j = 0; j<5; j++)
			used[j]=0;
		
		for (j = 21; j<24; j++)
			used[coloring->colors[j]]=1;
		
		for (j = 0; j<5; j++) {
			if (used[j]==0) {
				if (vertex1<0) vertex1=j;
				else vertex2=j;
			}
		}
		g_add_edge( mgraph, vertex1, vertex2 );
		
		g_set_data_both_directions( mgraph, 0, complement[indexes[0]][0], complement[indexes[0]][1] );
		g_set_data_both_directions( mgraph, 1, complement[indexes[1]][0], complement[indexes[1]][1] );
		g_set_data_both_directions( mgraph, 2, complement[indexes[2]][0], complement[indexes[2]][1] );
		g_set_data_both_directions( mgraph, 3, vertex1, vertex2 );
				
		g_add_edge( mgraph, 5+complement[indexes[3]][0], 5+complement[indexes[3]][1] );
		g_add_edge( mgraph, 5+complement[indexes[4]][0], 5+complement[indexes[4]][1] );
		g_add_edge( mgraph, 5+complement[indexes[5]][0], 5+complement[indexes[5]][1] );
		vertex1=-1; vertex2=-1;
		for (j = 0; j<5; j++)
			used[j]=0;
		
		for (j = 18; j<21; j++)
			used[coloring->colors[j]]=1;
		
		for (j = 0; j<5; j++) {
			if (used[j]==0) {
				if (vertex1<0) vertex1=j;
				else vertex2=j;
			}
		}
		g_add_edge( mgraph, 5+vertex1, 5+vertex2 );
		
		
		
		g_set_data_both_directions( mgraph, 4, 5+complement[indexes[3]][0], 5+complement[indexes[3]][1] );
		g_set_data_both_directions( mgraph, 5, 5+complement[indexes[4]][0], 5+complement[indexes[4]][1] );
		g_set_data_both_directions( mgraph, 6, 5+complement[indexes[5]][0], 5+complement[indexes[5]][1] );
		g_set_data_both_directions( mgraph, 7, 5+vertex1, 5+vertex2 );
		
		coloring_t *mcoloring = g_create_coloring( 2, 10 );
		mcoloring->colors[0] = 0;
		mcoloring->colors[1] = 0;
		mcoloring->colors[2] = 0;
		mcoloring->colors[3] = 0;
		mcoloring->colors[4] = 2;
		mcoloring->colors[5] = 2;
		mcoloring->colors[6] = 2;
		mcoloring->colors[7] = 2;
	

		g_dump_graphml_edgecolors( mgraph, mcoloring, i );
		printf("    </node>\n" );
		g_destroy( &mgraph );
		g_destroy_coloring( &mcoloring );
		
	}

	

	//print edges
	struct edge_t edge;
	for ( i=0; i<graph_of_colorings->num_vertices; i++ ) {
		edge = graph_of_colorings->vertices[i];
		if ( edge.vertex<0 ) continue;
		
		while( 1 ) {
			//print edge [a,b] only if a<b, so that there are not duplicate edges
			if (i<edge.vertex) {
				printf("    <edge id=\"e%lu:%lu\" source=\"n%lu\" target=\"n%lu\">\n", i, edge.vertex, i, edge.vertex);
				printf("      <data key=\"d1\">                                            \n");
				printf("    	<y:PolyLineEdge>                                         \n");
				printf("    	  <y:LineStyle type=\"line\" width=\"2.0\" color=\"#000000\"/> \n"); 
				printf("    	  <y:EdgeLabel></y:EdgeLabel>                            \n");
				printf("    	</y:PolyLineEdge>                                        \n");
				printf("      </data>                                                    \n");
				printf("    </edge>                                                      \n");
			}
			if (edge.next==NULL) break;
			edge = *(edge.next);
		};
		
	}
	printf( "\n" );

	
	printf("  </graph>\n" );
	printf("</graphml>\n" );
	
}





int main(int argc, char *argv[])
{

	int number_of_configuration;
	
	if (argc != 2) {
		printf( "USAGE: %s number_of_configuration\n", argv[0] );
		return 0;
	} 
	
	number_of_configuration = atoi( argv[1] );
	
	
	
	graph_t *graph = g_create( 26 );

	//create graph and enumerate edges so that edges neighbouring with central
	//vertices are the last

	int i, j, num_edges = 0;
	for (i = 0; i<6; i++ ) {
		for (j = 0; j<3; j++ ) {
			int vertex1 = i * 4 + 2;
			int vertex2 = vertex1 + j + 1;
			g_add_edge( graph, vertex1, vertex2 );
			g_set_data_both_directions( graph, num_edges++, vertex1, vertex2 );
		}
	}

	g_add_edge( graph, 0, 2 );
	g_add_edge( graph, 0, 6 );
	g_add_edge( graph, 0, 10 );
	
	g_add_edge( graph, 1, 14 );
	g_add_edge( graph, 1, 18 );
	g_add_edge( graph, 1, 22 );

	g_add_edge( graph, 0, 1 );
	
	g_set_data_both_directions( graph, num_edges++, 0, 2 );
	g_set_data_both_directions( graph, num_edges++, 0, 6 );
	g_set_data_both_directions( graph, num_edges++, 0, 10 );

	g_set_data_both_directions( graph, num_edges++, 1, 14 );
	g_set_data_both_directions( graph, num_edges++, 1, 18 );
	g_set_data_both_directions( graph, num_edges++, 1, 22 );

	g_set_data_both_directions( graph, num_edges++, 0, 1 );
	
	//create linegraph
	graph_t *linegraph = g_create_linegraph_from_enumerated_graph( graph, num_edges );
	
	graph_t *graph_of_colorings;	

	coloring_t *coloring = g_create_coloring( 5, linegraph->num_vertices );
	
	code_properties_t code_properties = g_code_properties( 5, linegraph->num_vertices ); //num_colors = 5

	
	int num_colorings=0;

	char **dictionary;
	
	int indexes[6] = {0, 0, 0, 0, 0, 0};
	
	int configurations[56][6] = {
		{9,8,5,7,6,5},
		{9,8,7,6,1,1},
		{9,8,8,8,7,6},
		{9,8,8,8,7,1},
		{9,8,7,8,8,1},
		{9,8,5,8,7,7},
		{9,8,8,7,2,1},
		{9,9,8,7,2,1},
		{9,8,7,2,1,0},
		{9,9,8,8,7,1},
		{9,9,5,8,7,2},
		{9,9,5,7,6,1},
		{9,8,8,8,7,2},
		{9,9,2,8,7,6},
		{9,8,2,9,5,1},
		{9,5,2,4,3,2},
		{9,8,0,8,7,2},
		{9,7,2,8,7,2},
		{9,8,4,8,7,2},
		{9,5,0,8,7,2},
		{9,8,7,8,7,2},
		{9,5,4,8,7,2},
		{9,8,4,6,2,0},
		{9,8,4,9,7,3},
		{9,8,7,9,5,3},
		{9,8,5,9,9,0},
		{9,8,7,6,4,0},
		{9,8,4,7,6,3},
		{9,8,7,9,6,5},
		{9,8,5,8,7,6},
		{9,8,5,8,7,1},
		{9,8,5,7,1,1},
		{9,8,5,9,8,4},
		{9,8,5,9,8,7},
		{9,8,5,9,9,2},
		{9,8,5,9,8,0},
		{9,8,5,7,6,3},
		{9,7,5,2,1,0},
		{9,8,5,7,4,1},
		{9,8,5,8,4,0},
		{9,9,2,8,7,2},
		{9,8,5,8,7,0},
		{9,8,5,9,7,2},
		{9,8,5,8,7,3},
		{9,9,8,7,6,5},
		{9,9,2,7,6,2},
		{9,9,5,7,6,5},
		{9,8,0,6,0,0},
		{9,9,2,8,7,1},
		{9,9,2,8,6,0},
		{9,9,8,7,6,2},
		{9,9,5,8,7,1},
		{9,9,8,8,7,6},
		{9,9,5,7,2,1},
		{9,8,8,7,6,2},
		{9,9,5,8,7,6}
	};
	
				  
	//set current indexes
	for (j = 5; j>=0; j-- ) {
		indexes[j] = configurations[number_of_configuration][j];
	}
	
	indexes[0] = 9;
	indexes[1] = 9;
	indexes[2] = 1;
	indexes[3] = 4;
	indexes[4] = 6;
	indexes[5] = 3;
	
	set_prefix( coloring, 6, num_edges, indexes );

	//the coloring is correct when the colors of edges around central vertex are 0,1,2,3
	int correct_coloring = 0;

	num_colorings = 0;
	while( g_get_next_coloring_offset( linegraph, coloring, num_edges-7 ) ) {
		if ( ( coloring->colors[num_edges-7] == 0 ) &&
			( coloring->colors[num_edges-6] == 1 ) &&
			( coloring->colors[num_edges-5] == 2 ) &&
			( coloring->colors[num_edges-4] == 3 ) &&
			( coloring->colors[num_edges-3] == 0 ) &&
			( coloring->colors[num_edges-2] == 1 ) ) {
			correct_coloring = 1;
		}
		num_colorings++;
	};

	set_prefix( coloring, 6, num_edges, indexes );
	
	dictionary = g_create_dictionary( num_colorings );
	graph_of_colorings = g_create( num_colorings );

	num_colorings=0;
	while ( g_get_next_coloring_offset( linegraph, coloring, num_edges-7 ) ) {
		dictionary[num_colorings] = g_create_code( &code_properties );
		g_coloring_to_code( coloring, dictionary[num_colorings], &code_properties );
		num_colorings++;
	}	

	int m;
	//perform switching
	for (m=0; m<num_colorings; m++) {
		g_code_to_coloring( dictionary[m], coloring, &code_properties );
		perform_kempe_switching_on_coloring( linegraph, graph_of_colorings, dictionary, coloring, num_edges-7 );
	}
	
	set_prefix( coloring, 6, num_edges, indexes );
	//g_get_next_coloring_offset( linegraph, coloring, num_edges-7 );	
	//g_dump_tgf_edgecoloring( graph, coloring );
//	g_dump_goc_mg_graphml( graph, graph_of_colorings, linegraph, dictionary, 5, indexes ); //num_colors =  5
	g_dump_goc_graphml( graph, graph_of_colorings, linegraph, dictionary, 5 ); //num_colors =  5

	for (m=0; m<num_colorings; m++) {
		free(dictionary[m]);
	}
	g_destroy_dictionary( &dictionary );
	
	g_destroy( &graph_of_colorings );
	
	g_destroy( &graph );
	g_destroy( &linegraph );
	g_destroy_coloring( &coloring );
	
	return 0;
}
