#include "graph_lib.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <math.h>
#include <time.h>

#define LOG_DEBUG 1
#define LOG_WARN 2
#define LOG_ERR 3
#define LOG_LEVEL LOG_WARN

double diffclock(clock_t clock1,clock_t clock2)
{
	double diffticks=clock2-clock1;
	double diffms=(diffticks*1000)/CLOCKS_PER_SEC;
	return diffms;
} 



int main(int argc, char *argv[])
{
	int num_vertices, max_degree, num_colors; 
	
	/* dont know how to put attributes when invoking by gdb, hence this hack
	argc=3;
	argv[1]="8";
	argv[2]="5";
	 */
	
	if (argc != 3) {
		printf( "USAGE: ./graph num_vertices max_degree\n" );
		return 0;
	} else {
		num_vertices = atoi( argv[1] );
		max_degree = atoi( argv[2] );
		if (max_degree<1 || num_vertices<1) {
			printf( "       arguments must be positive integers" );
			return 0;
		}
	}

	char input_file_name[60];
	sprintf( input_file_name, "input/vert%ideg%i.g6", num_vertices, max_degree );
	printf( "opening %s\n", input_file_name );	
	FILE *input_file = fopen( input_file_name, "r" );
	if (NULL == input_file ) {
		msg (LOG_ERR, "failed to open input file for reading ");
		exit(1);	
	}; 
	
	num_colors = max_degree+1;
	
	char output_file_name[60];
	sprintf( output_file_name, "output/goc_vert%ideg%icol%i.g6", num_vertices, max_degree, num_colors );
	FILE *output_file = fopen( output_file_name, "w" );
	if (NULL == output_file ) {
		msg (LOG_ERR, "failed to open file for writing ");
		exit(1);	
	}; 

	char output_meta_file_name[60];
	sprintf( output_meta_file_name, "output/meta_vert%ideg%icol%i.g6", num_vertices, max_degree, num_colors );
	FILE *output_meta_file = fopen( output_meta_file_name, "w" );
	if (NULL == output_meta_file ) {
		msg (LOG_ERR, "failed to open metafile for writing ");
		exit(1);	
	}; 

	fprintf( output_meta_file, "components\n"  );



	graph_t *graph, *linegraph, *graph_of_colorings; 
	coloring_t *coloring;
	code_properties_t code_properties;
	unsigned long num_colorings = 0;
	char **dictionary;
	
	long count_graphs = 0;
	
	clock_t begin, end;
	
	//loop for all file lines (i.e. all graphs
	while( 	NULL != ( graph = g_create_from_file( input_file ) ) ) { 		

		printf( "graph#: %ld \t num_vertices: %d \t max_degree: %d \n", count_graphs++, num_vertices, max_degree );

		
		/*/skip graphs with maxdegree less than "max_degree"
		printf( "max degree: %d\n", g_max_degree( graph ) );
		if (g_max_degree( graph ) < max_degree) {
			g_destroy( &graph );
			continue;
		}
		*/
		
		linegraph = g_create_linegraph( graph );
		
		
		/*/skip graphs that can be colored with max_degree+1 colors
		coloring = g_create_coloring( max_degree+1, linegraph->num_vertices );		
		g_init_coloring_for_regular_graph( coloring, max_degree );
		if ( g_get_next_coloring_offset( linegraph, coloring, max_degree )) {
//			g_dump_coloring( coloring );
			g_destroy_coloring( &coloring );
			g_destroy( &linegraph );
			printf(" skipping class 1 graph\n" );
			continue;
		}
		g_destroy_coloring( &coloring );
		*/		
		




		nv_t i;
		
		coloring = g_create_coloring( num_colors, linegraph->num_vertices );		
		
		code_properties = g_code_properties( num_colors, linegraph->num_vertices );
		
		//count number of colorings
		num_colorings = 0;
		

		//this loop checks graph sign equivalence
		//and computes number of colorings
		
//		g_enumerate_edges( graph );
//		int sign = 0, sign_equivalence = 1;

		g_init_coloring_for_regular_graph( coloring, max_degree );
		
		while (g_get_next_coloring_offset( linegraph, coloring, max_degree ) ) {
			
			//compute initial sign
/*			if (sign == 0) {
				sign = g_get_graph_sign( graph, coloring ); 
				
			} else {
				//if signs don't match, set unset sign_equal
				if ( ( sign_equivalence == 1 )
					&& ( sign != g_get_graph_sign( graph, coloring ) ) ) {
					sign_equivalence = 0;
				}; 
			}
*/
			num_colorings++;
//			g_dump_dot_edgecoloring( graph, coloring );
//			g_dump_coloring( coloring );

		}


		printf("num_colorings: %lu\n", num_colorings );
		/*
		printf("sign_equivalence: %d\n", sign_equivalence );
		printf("bipartiteness: %d\n", g_bipartiteness( graph ) );
		printf("regularity: %d\n", g_regularity( graph ) );
		*/
		
		//create graph of colorings
		graph_of_colorings = g_create( num_colorings );
	
		//store all colorings to dictionary
		g_init_coloring_for_regular_graph( coloring, max_degree );
		

		
		dictionary = g_create_dictionary( num_colorings );
		num_colorings=0;
		while (g_get_next_coloring_offset( linegraph, coloring, max_degree ) ) {
			dictionary[num_colorings] = g_create_code( &code_properties );
			g_coloring_to_code( coloring, dictionary[num_colorings], &code_properties );
			num_colorings++;
		}	

			
		begin = clock();
		
		//perform switching
		for (i=0; i<num_colorings; i++) {
			g_code_to_coloring( dictionary[i], coloring, &code_properties );
			perform_kempe_switching_on_coloring( linegraph, graph_of_colorings, dictionary, coloring, max_degree );
		}
		end = clock();
		printf("switching took: %f\n", diffclock(begin, end) );
		

		begin = clock();
		
		//store graph_of_colorings to file!
//		g_write_to_file_sparse( graph_of_colorings, output_file );	

		end = clock();
		printf("writing to sparse took: %f\n", diffclock(begin, end) );

		
//		fprintf( output_meta_file, "%lu\t%d\n", g_num_components( graph_of_colorings ), sign_equivalence  );
		fprintf( output_meta_file, "%lu\n", g_num_components( graph_of_colorings ) );
		
		for (i=0; i<num_colorings; i++) {
			free(dictionary[i]);
		}
		g_destroy_dictionary( &dictionary );
		g_destroy_coloring( &coloring );
		g_destroy( &graph );
		g_destroy( &linegraph );
		g_destroy( &graph_of_colorings );
}
	
	fclose( input_file );
	fclose( output_file );
	fclose( output_meta_file );
	
	
	printf("alles gute\n\n\n");
	
	return 0;

}
