//#include <stdio.h>
#include "graph_lib.h"
#include <stdlib.h>

void g_dump_coloring_from_to( coloring_t *coloring, nv_t from, nv_t to ) {
	nv_t i;
	if (to > coloring->num_vertices) {
		msg( LOG_ERR, "to > coloring->num_vertices" );
		exit(0);
	}
	if (from >= coloring->num_vertices) {
		msg( LOG_ERR, "from >= coloring->num_vertices" );
		exit(0);
	}
	
	
	for ( i=0; i<to-from; i++ ) {
		printf( "%2d ", coloring->colors[i+from] );
		if ((i+1)%50==0) printf("\n");
	}
	printf( "\n" );
}


nv_t g_get_length_of_kempe_path( coloring_t *coloring, 
					graph_t *graph, nv_t vertex, color_t new_color )
{
	
	color_t color1 = new_color, color2 = coloring->colors[vertex];
//	printf( "initial_vertex: %d,   color1 = %d    color2 = %d \n", vertex, color1, color2 );
	
	nv_t queue[coloring->num_vertices];
	nv_t mark[coloring->num_vertices];
	nv_t i, position, num_queued=0, current_vertex;
	edge_t *edge;
	edge_iterator_t iterator;
	
	for( i=0; i<coloring->num_vertices; i++ ) {
		queue[i] = -1;
		mark[i] = 0;
	}


	//put first vertex to the queue
	position=0;
	num_queued=0;
	queue[position+num_queued] = vertex;
	num_queued++;
	
	while ( num_queued>0 ) {
		
		current_vertex = queue[position]; //pop vertex from queue
		num_queued--;
		position++;
		
		mark[current_vertex] = 1; //mark it for change
		
		iterator = g_new_edge_iterator(graph); //get all neighbors of current vertex
		iterator.vertex = current_vertex;
	
		//add to queue - each neighbor not yet marked
		while ( NULL!= (edge = g_edge_iterator_next_for_vertex( &iterator ) ) ) {
			if (mark[edge->vertex]==0 && ( coloring->colors[edge->vertex]==color1 || coloring->colors[edge->vertex]==color2 ) ) { 
				//add vertex to queue
				queue[position+num_queued] = edge->vertex;
				num_queued++;
			} else {
				//vertex is already marked for switch or wrong color
				// so we don't do anything 
			}
		}
	}
	
	nv_t length = 0;
	
	for ( i=0; i<coloring->num_vertices; i++) {
		if (mark[i]==1) {
			length++;
		}
	}
	
	return length;
}


char color_names[][10] = { "red", "green", "blue", "yellow", "black", "cyan", "grey", "violet", "lightblue", "maroon" };






int enumerate_complete_graph( graph_t *orig_graph ) {
	int i, j, n = orig_graph->num_vertices/2;
	int edge_count = 0;
	
	//manually label (enumerate) edges in original graph K_2n
	for ( i=0; i < n-1; i++ ) {
		for ( j=i+1; j < n; j++) {
			g_set_data_both_directions( orig_graph, edge_count, i, j );
			edge_count++;
		}
	}

	for ( i=0; i < n-1; i++ ) {
		for ( j=i+1; j < n; j++) {
			g_set_data_both_directions( orig_graph, edge_count, n+i, n+j );
			edge_count++;
		}
	}
	
	//direct connections	
	for ( i=0; i<n; i++ ) {
		g_set_data_both_directions( orig_graph, edge_count, i, n+i );
		edge_count++;
	}

	//indirect connections	
	for ( i=0; i<n; i++ ) {
		for ( j=1; j<n; j++ ) {
			g_set_data_both_directions( orig_graph, edge_count, i, n + ((i+j)%n) );
			edge_count++;
		}
	}

	return edge_count;
}


int main(int argc, char *argv[]) {	
	
	//zacnem s 2 grafmi K_n
	//postupne je: 
	//   n nad 2 hran Kn
	//   dtto v druhom grafe
	//   n priamych spajajucich hran
	//   n*(n-1) sikmych spajajucich hran
	// dostanem K_(2n+2) = 2n nad 2
	

	//orig coloring su najprv farby hran (v prirodzenom poradi) grafu Kn a potom farby trciacich hran.
	//int n = 3;
	//int orig_coloring[] = { 0, 1, 2, 2, 1, 0 };


	//generated from n=3
	int n = 5;
	int orig_coloring[] = { 0, 1, 2, 3, 2, 4, 1, 3, 4, 0, 4, 3, 0, 1, 2};

	//generated from n=5
	//int n = 9;
	//int orig_coloring[] = { 0, 1, 2, 3, 4, 5, 6, 7, 2, 4, 1, 7, 3, 8, 6, 3, 4, 8, 6, 0, 5, 0, 5, 8, 7, 1, 6, 7, 5, 8, 0, 1, 2, 2, 4, 3, 8, 5, 7, 6, 2, 3, 1, 4, 0};


	
//	int n = 5;
//	int orig_coloring[] = { 0, 2, 4, 3, 1, 2, 4, 3, 0, 1, 1, 3, 4, 0, 2 };

//	int n = 9;
//	int orig_coloring[] = { 0, 2, 4, 3, 1, 5, 6, 7, 1, 2, 4, 6, 3, 7, 8, 3, 0, 5, 8, 4, 6, 1, 8, 7, 5, 0, 7, 6, 8, 5, 0, 2, 4, 1, 2, 3, 8, 5, 7, 6, 2, 3, 4, 0, 1};
	
	//create graph K_2n
	graph_t *orig_graph = g_create( 2 * n );

	int i, j, k;

	for ( i=0; i<2*n-1; i++ ) {
		for ( j=i+1; j<2*n; j++ ) {
			g_add_edge( orig_graph, i, j );
		}
	}

	int edge_count = enumerate_complete_graph( orig_graph );

	
	//vertices 0..n-1 induce subgraph Kn
	//and vertices n..2n-1 induce second one
	
	
	graph_t *graph = g_create_linegraph_from_enumerated_graph( orig_graph, edge_count );
	enumerate_complete_graph( orig_graph );
	
	coloring_t *coloring = g_create_coloring( 2 * n - 1, graph->num_vertices );

	int offset = n*(n-1)/2;

	
	for ( i=0; i<offset; i++ ) {
		coloring->colors[i] = orig_coloring[i];
		coloring->colors[i + offset] = orig_coloring[i];
	}

	for ( i=0; i<n; i++ ) {
		coloring->colors[ 2*offset + i ] = orig_coloring[offset + i];
	}	

	offset = 2*offset + n;
	

	for ( i=0; i<n-1; i++ ) {
		coloring->colors[ offset ] = n+i;
		offset++;
	}
/*
	printf("%d\n", offset );
	printf("%d\n", graph->num_vertices );
	
		
	int default_coloring[] = {9, 11, 12, 13, 14, 10, 15, 16, 12, 9, 15, 13, 16, 11, 14, 10, 10, 14, 16, 15, 13, 9, 12, 11, 16, 12, 13, 14, 15, 11, 10, 9, 10, 11, 9, 12, 14, 16, 13, 15, 12, 16, 10, 13, 14, 15, 11, 9, 10, 13, 9, 15, 16, 14, 12, 11, 11, 16, 12, 14, 13, 10, 15, 9, 15, 13, 10, 16, 11, 9, 14, 11};//posledne ma byt 12, ale je 11, aby to get_next_coloring zmenil na 12
	
	for ( i=0; i<72; i++ ) {
		coloring->colors[ offset+i ] = default_coloring[i];
	}	
*/	
	int ok;
	while (1 == g_get_next_coloring_offset( graph, coloring, offset ) ) {
			g_dump_coloring_from_to( coloring, 0, coloring->num_vertices );
//exit(0);
		
		ok=1;

		//this checks lengths of 1-factors. Tries to find longest alterning paths starting in every edge of K_n
		//and every color from the set of added edges (indirectly connecting ones)
		for (i = 0; i<n-1; i++ ) {
			for (j=n; j<2*n-1; j++) {
				if (g_get_length_of_kempe_path( coloring, graph, i, j ) != 2*n ) {
					printf( "%lu != %d\n", g_get_length_of_kempe_path( coloring, graph, i, j ), 2*n );
					printf( "vertex: %d,   color: %d\n", i, j );

					ok = 0;
//					exit(1);
					break;
				}
			}
			if (ok==0) break;
		}
		
		
		if (ok) {
			printf(" coloring found\n" );
			g_dump_coloring( coloring );
			int edge_num;

			printf("\ninput for next iteration: \n");
			printf("int n = %d;\n", 2*n-1);
			printf("int orig_coloring[] = {");
			
			for ( i=0; i<2*n-2; i++ ) {
				for ( j=i+1; j<2*n-1; j++ ) {
					edge_num = g_get_data( orig_graph, i, j );
					printf(" %d,", coloring->colors[edge_num] );
				}
			}

			int color_ok=0;
			
			for ( i=0; i<2*n-1; i++ ) {
				for ( j=0; j<2*n-1; j++ ) {
					color_ok = 1;
					for ( k=0; k<2*n-1; k++ ) {
						if( k!=i ) {
							edge_num = g_get_data( orig_graph, i, k );
							if( coloring->colors[edge_num] == j) {
								color_ok=0;
								break;
							}
						}
					}
					
					if (color_ok == 1 ) {
						printf( " %d", j);
						break;
					}
				}
				if( i<2*n-2 ) printf(",");
			}			
			printf("};\n\n");
			
			/*
			for ( i=0; i<2*n-1; i++ ) {
				for ( j=i+1; j<2*n; j++ ) {
					edge_num = g_get_data( orig_graph, i, j );
					printf("%d -- %d [color=%s];\n", i, j, color_names[coloring->colors[edge_num]] );
				}
			}
			*/			
			
			exit(0);
		}
	}

	
	printf("nothing found\n");
	
	g_destroy( &orig_graph );
	g_destroy( &graph );


	
	return 0;
}
